Package: latte
Type: Package
Title: Interface to 'LattE' and '4ti2'
Version: 0.2.2
Authors@R: c(person("David", "Kahle", email = "david@kahle.io", role = c("aut", "cph", "cre"), comment = c(ORCID = "0000-0002-9999-1558")),
	person("Luis", "Garcia",email = "lgarcia@shsu.edu", role = c("aut", "cph")),
	person("Ruriko", "Yoshida", email = "ryoshida@nps.edu", role = c("aut", "cph")))
Maintainer: David Kahle <david@kahle.io>
Description: Back-end connections to 'LattE' (<https://www.math.ucdavis.edu/~latte/>) 
	for counting lattice points and integration inside convex polytopes and 
	'4ti2' (<http://www.4ti2.de/>) for algebraic, geometric, and combinatorial 
	problems on linear spaces and front-end tools facilitating their use in the 
	'R' ecosystem.
License: GPL-2
URL: https://github.com/dkahle/latte
BugReports: https://github.com/dkahle/latte/issues
SystemRequirements: LattE <https://www.math.ucdavis.edu/~latte/>, 4ti2
        <http://www.4ti2.de/>
Imports: magrittr, stringr, mpoly, ggplot2, memoise, dplyr, usethis,
        glue
Suggests: knitr, rmarkdown
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-06-09 05:07:18 UTC; david_kahle
Author: David Kahle [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0002-9999-1558>),
  Luis Garcia [aut, cph],
  Ruriko Yoshida [aut, cph]
Repository: CRAN
Date/Publication: 2025-06-09 05:20:02 UTC
Built: R 4.6.0; ; 2026-04-29 20:46:05 UTC; unix
