# Generated by roxygen2: do not edit by hand

S3method(print,tableau)
export(edit_r_environ)
export(fgraver)
export(fgroebner)
export(fhilbert)
export(fmarkov)
export(format_latte)
export(fzbasis)
export(genmodel)
export(get_4ti2_path)
export(get_latte_path)
export(graver)
export(groebner)
export(has_4ti2)
export(has_latte)
export(hilbert)
export(kprod)
export(latte_count)
export(latte_fcount)
export(latte_max)
export(latte_min)
export(markov)
export(missing_4ti2_stop)
export(missing_latte_stop)
export(ones)
export(plot_matrix)
export(ppi)
export(qsolve)
export(read.latte)
export(read_latte)
export(set_4ti2_path)
export(set_latte_path)
export(tab2vec)
export(tableau)
export(vec2tab)
export(write.latte)
export(write_latte)
export(zbasis)
export(zsolve)
import(mpoly)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(glue,glue)
importFrom(magrittr,"%>%")
importFrom(memoise,memoise)
importFrom(stats,runif)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(stringr,str_trim)
importFrom(stringr,str_which)
importFrom(usethis,edit_r_environ)
importFrom(utils,download.file)
